package org.greenstone.gsdl3.util;

import java.util.HashMap;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/** GSCGI holds the cgi args / xml parameters fro cgi type requests to the receptionist
 * it includes methods to convert long parameter names to short cgi arg names
 */
public class GSCGI {

    // the args that are used in the interface
    public static final String ACTION_ARG = "a"; // the major type of action- eg query or browse or process
    public static final String SUBACTION_ARG = "sa"; // subtype of action if we want different processing than the default
    public static final String REQUEST_TYPE_ARG = "rt"; // whether the request is just to display the service form, or to actually do a request to teh service
    public static final String REQUEST_ONLY_ARG = "ro"; // if == 1 do the request and pass back the xml - no page formatting
    public static final String OUTPUT_ARG = "o"; // if processing is to be done, what type of output - html/xml/other??
    public static final String SERVICE_ARG = "s"; // the name of the service
    public static final String CLUSTER_ARG = "c";   // these two are the same
    public static final String COLLECTION_ARG = "c"; 
    public static final String LANGUAGE_ARG = "l";
    public static final String DOCUMENT_ARG = "d";
    public static final String RESOURCE_ARG = "r";
    public static final String PROCESS_ID_ARG = "pid"; // if a request wasn't completed, this identifies the request - used when asking for a status update
    
    /** long to short mapping */
    protected HashMap long_map_=null;
    /** short to long mapping */
    protected HashMap short_map_=null;
    
    /** initialises the maps with known parameters */
    public GSCGI() {
	long_map_ = new HashMap();
	short_map_ = new HashMap();

	// initialize with the ones we know about
	addStaticParam(ACTION_ARG); 
	addStaticParam(SUBACTION_ARG); 
	addStaticParam(REQUEST_TYPE_ARG);
	addStaticParam(REQUEST_ONLY_ARG);
	addStaticParam(CLUSTER_ARG); 
	addStaticParam(COLLECTION_ARG); 
	addStaticParam(LANGUAGE_ARG); 
	addStaticParam(DOCUMENT_ARG); 
	addStaticParam(RESOURCE_ARG); 
	addStaticParam(OUTPUT_ARG); 
	addStaticParam(SERVICE_ARG); 
	addStaticParam(PROCESS_ID_ARG); 
    }

    /* static params - have only one form */
    public boolean addStaticParam(String param) {
	return addParam(param, param);
    }
    /** adds another param to teh list 
     * should check that its not already present? */
    public boolean addParam(String long_name, String short_name) {
	long_map_.put(long_name, short_name);
	short_map_.put(short_name, long_name);
	return true;
    }
	
    /** takes a paramList element  - contains <param name="x"> elements
     * converts all the short names to long names 
     * returns true if successful, false otherwise */
    public boolean paramListToLongNames(Element param_list) {
	
	if (!param_list.getNodeName().equals(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER)) {
	    System.err.println("GSCGI Error: wrong element passed to toLong");
	    return false;
	}
	NodeList params = param_list.getElementsByTagName(GSXML.PARAM_ELEM);
	for (int i=0; i< params.getLength(); i++) {
	    Element param = (Element)params.item(i);
	    String name = param.getAttribute(GSXML.NAME_ATT);
	    param.setAttribute(GSXML.NAME_ATT, toLong(name));
	}
	return true;

    }
    /** adds shortnames to the params in the list */
    public boolean paramListAddShortNames(Element param_list) {
	if (!param_list.getNodeName().equals(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER)) {
	    System.err.println("GSCGI Error: wrong element passed to addShortNames");
	    return false;
	}
	
	NodeList params = param_list.getElementsByTagName(GSXML.PARAM_ELEM);
	for (int i=0; i< params.getLength(); i++) {
	    Element param = (Element)params.item(i);
	    String name = param.getAttribute(GSXML.NAME_ATT);
	    param.setAttribute(GSXML.PARAM_SHORTNAME_ATT, toShort(name));
	}
	return true;

    }
    
    /** returns the long form of a short arg - 
     * returns the original if it doesn't know about this short arg */
    public String toLong(String short_name) {
	
	if (short_map_.containsKey(short_name)) {
	    return (String)short_map_.get(short_name);
	}
	return short_name; // just return the original
    }
    /** returns the short form of a long arg */
    public String toShort(String long_name) {

	if (long_map_.containsKey(long_name)) {
	    return (String)long_map_.get(long_name);
	}

	// else need to create a new mapping
	String short_name = newShortName(long_name);
	long_map_.put(long_name, short_name);
	short_map_.put(short_name, long_name);
	return short_name;
    }
    
    /** tries to create a new short form of the long name - 
     * if it cant, it just returns the original name */
    protected String newShortName(String long_name) {
	
	// for now, try the first letter, then first two etc.
	int i=1;
	String short_try= long_name.substring(0,1);
	while(short_map_.containsKey(short_try)&& i<long_name.length()) {
	    i++;
	    short_try = long_name.substring(0, i);
	}
	return short_try;
	
    }
}


