/*
 *    SimpleServer.java
 *    Copyright (C) 2000 Stuart Yeates 
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// the package we're in
package org.nzdl.gsdl.service;

// java standard library classes used
import java.io.IOException;
import java.io.Serializable;
import java.lang.Cloneable;
import java.util.Set;
import java.util.Iterator;

import org.omg.CORBA.ORB;

// local libraries
import org.nzdl.gsdl.corba.gsdlInterface.*;
import org.nzdl.gsdl.service.NzdlCollectionInfo; 
import org.nzdl.gsdl.service.NzdlQuery;
import org.nzdl.gsdl.service.NzdlRequest;
import org.nzdl.gsdl.service.NzdlResponse;
import org.nzdl.gsdl.service.NzdlResultSet;
import org.nzdl.gsdl.service.NzdlService;
import org.nzdl.gsdl.util.NzdlCorbaFactory;


/**
 * Act as a Server.
 *
 * Designed as a mirror image of NzdlServiceClient
 *
 * @author stuart yeates (say1@cs.waikato.ac.nz)
 * @version $Revision: 2284 $
 * @see org.nzdl.gsdl.service.NzdlService
 * @see org.nzdl.gsdl.service.NzdlServiceClient
 * @see org.nzdl.gsdl.service.NzdlServiceWrapper
 * @see org.nzdl.gsdl.corba.gsdlInterface._corbaifaceImplBase
 * 
 */


public class NzdlServiceServer extends _corbaifaceImplBase {

  /** The underlying interface */
  NzdlService nzdl = null;
  
  /** The normal constructor */
  public NzdlServiceServer(NzdlService service) {
    nzdl = service;
  }

  /** The beans constructor */
  protected NzdlServiceServer() {
    nzdl = null;
  }

  /** initialisation (not guaranteed to be called ...) */
  public boolean initialise (corbaComErrorHolder error) {
    // TODO should map the anything in NzdlService?
    if (nzdl == null)
      throw new Error ("NzdlServer needs a NzdlService to foward calls to ...");
    error.value = corbaComError.from_int(corbaComError._corbaNoError);
    return true;
  }
  
  /**
   * Allow the client to configure a CORBA server.
   * @param key the parameter to be set.
   * @param values what that parameter will be set to.
   */
  public void configure (corbatext_t key, 
			 corbatext_t[] cfgline,
			 corbaComErrorHolder error) {
    //      m_nzdlServer.configure( NzdlCorbaFactory.toCorbaText( _key ),
    //  			    NzdlCorbaFactory.toCorbaTextArray( _values ));

    nzdl.configure( NzdlCorbaFactory.toString(key),
		    NzdlCorbaFactory.toSet(cfgline));
    error.value = corbaComError.from_int(corbaComError._corbaNoError);
  }


  /**
   * Obtain the set of collection names for a Greenstone library.
   * @param collist A set of collection names in string format.
   * @param error a corba error object
   */
  public void collectionList (corbatext_tarrayHolder collist,
			      corbaComErrorHolder error) {
    System.err.println("Server::getCollectionSet() start " + error.value.value());
    //corbatext_tarrayHolder c_collSet = NzdlCorbaFactory.createCorbaTextArrayHolder();
    //m_nzdlServer.collectionList(c_collSet);
    //Set collSet = new HashSet();
    //for (int i=0; i<c_collSet.value.length; i++) {
    //  collSet.add( NzdlCorbaFactory.toString( c_collSet.value[i] ) );
    //}
    //return collSet ;
    Set set = nzdl.getCollectionSet();
    System.err.println("Server::getCollectionSet() in " + error.value.value());
    System.err.println("Server::getCollectionSet() in " + set);

    collist.value = new corbatext_t[set.size()];

    Iterator itr = set.iterator();
    int i = 0;
    while (itr.hasNext()) {
      String s = (String) itr.next();
      if (s.indexOf("nuipepa") == -1)
	collist.value[i++] = NzdlCorbaFactory.toCorbaText(s);
    }
    error.value = corbaComError.from_int(corbaComError._corbaNoError);
    System.err.println("Server::getCollectionSet() end " + error.value.value());

    return;
  }

  /**
   * Does the server have a specific collections? 
   * @param has a boolean
   * @param error a corba error object
   */
  public void hasCollection (corbatext_t corbaCollection,
			     org.omg.CORBA.BooleanHolder has, 
			     corbaComErrorHolder error) {
    //      corbaComErrorHolder c_err = NzdlCorbaFactory.createComErrorHolder();
    //      org.omg.CORBA.BooleanHolder c_has = new org.omg.CORBA.BooleanHolder();
    //      m_nzdlServer.hasCollection( NzdlCorbaFactory.toCorbaText(_name), c_has, c_err );
    //      if (c_err.value.value() != 0)
    //        System.err.println("hasCollection error " + c_err.value.value());
    //      return c_has.value;
    String name = NzdlCorbaFactory.toString(corbaCollection);
    has.value  = nzdl.hasCollection(name);
    error.value = corbaComError.from_int(corbaComError._corbaNoError);
    return;
  }

  /**
   * Is the collection avalibale for use?
   * @param error a corba error object
   * @return <tt>true</tt> if the collection is avaliable, 
   *    <tt>false</tt> otherwise
   */
  public void ping (corbatext_t collection, 
		    org.omg.CORBA.BooleanHolder has, 		    
		    corbaComErrorHolder error) {
    String name = NzdlCorbaFactory.toString(collection);
    has.value = nzdl.hasCollection(name);
  }

  /**
   * Get a document
   * @param collection the collection the document is in
   * @param request the details of the document
   * @param response the document
   * @param error a corba error object
   */
  public void getDocument (corbatext_t collection, 
			   corbaDocRequestHolder request, 
			   corbaDocResponseHolder response, 
			   corbaComErrorHolder error) {
    //      corbatext_t c_name = NzdlCorbaFactory.toCorbaText( _name );
    //      corbaDocRequestHolder c_request = NzdlCorbaFactory.createDocRequestHolder( _docID );
    //      corbaDocResponseHolder c_response = NzdlCorbaFactory.createDocResponseHolder( );
    //      corbaComErrorHolder c_err = NzdlCorbaFactory.createComErrorHolder();
    //      m_nzdlServer.getDocument( c_name, c_request, c_response, c_err );
    //      if (c_err.value.value() != 0)
    //        System.err.println("getDocument() error " + c_err.value.value());
    //      return NzdlCorbaFactory.toString( c_response.value.doc );
    String collectionName = NzdlCorbaFactory.toString(collection);
    String documentID = NzdlCorbaFactory.toString(request.value.OID);
    String doc = nzdl.getDocument( collectionName, documentID );
    response.value.doc = NzdlCorbaFactory.toCorbaText(doc);
    error.value = corbaComError.from_int(corbaComError._corbaNoError);
    return;
  }

  public void getCollectInfo (corbatext_t collection, 
			      corbaColInfoResponseHolder response, 
			      corbaComErrorHolder error) {
    //      corbaComErrorHolder c_err = NzdlCorbaFactory.createComErrorHolder();
    //      corbaColInfoResponseHolder c_info = NzdlCorbaFactory.createColInfoResponseHolder();
    //      m_nzdlServer.getCollectInfo( NzdlCorbaFactory.toCorbaText(_name), c_info, c_err );
    //      if (c_err.value.value() != 0)
    //        System.err.println("getCollectionInfo() error " + c_err.value.value());
    //      return new NzdlCollectionInfo( c_info.value );
    String collectionName = NzdlCorbaFactory.toString(collection);
    NzdlCollectionInfo info = nzdl.getCollectionInfo( collectionName );
    response.value = info.getInfo();
    error.value = corbaComError.from_int(corbaComError._corbaNoError);
    return;
  }

  public void getFilterInfo (corbatext_t collection, 
			     corbatext_tarrayHolder filterNames, 
			     corbaComErrorHolder error) {
    String collectionName = NzdlCorbaFactory.toString(collection);
 
    Set set = nzdl.getFilterSet(collectionName);

    Iterator itr = set.iterator();
    int i = 0;
    while (itr.hasNext()) {
      String s = (String) itr.next();
      filterNames.value[i++] = NzdlCorbaFactory.toCorbaText(s);
    }

    error.value = corbaComError.from_int(corbaComError._corbaNoError);
    return;
  }

  public void getFilterOptions (corbatext_t collection, 
				corbatext_t option, 
				corbaFilterOptionsResponseHolder response, 
				corbaComErrorHolder error) {
    String collectionName = NzdlCorbaFactory.toString(collection);
    System.err.println("TO BE IMPLEMENTED: DON'T KNOW HOW YET !!! (getFilterOptions)");

    error.value = corbaComError.from_int(corbaComError._corbaProtocolError);
    return;
  }

  public  void filter (corbatext_t collection, 
		       corbaFilterRequest request, 
		       corbaFilterResponseHolder response, 
		       corbaComErrorHolder error) {
    //corbatext_t c_name = NzdlCorbaFactory.toCorbaText( _name );
    //corbaComErrorHolder c_err = NzdlCorbaFactory.createComErrorHolder();
    //m_nzdlServer.filter( c_name, _request.getFilter(), _response.getHolder(), c_err );
    //if (c_err.value.value() != 0)
    //  System.err.println("service() error " + c_err.value.value());
    error.value = corbaComError.from_int(corbaComError._corbaNoError);

    String collectionName = NzdlCorbaFactory.toString(collection);
    NzdlRequest _request = new NzdlRequest(request);
    NzdlResponse _response = new NzdlResponse();
    _response.setResponse(response.value);

    try {
      nzdl.service(collectionName,_request,_response);
    } catch (Throwable t) {
          error.value = corbaComError.from_int(corbaComError._corbaProtocolError);
    }
    return;
  }
}


